/*! \file msg.h
    \brief Message queue interface.
*/

#ifndef _MSG_H
#define _MSG_H



// the known events
// ToDo: this can be application specific, no need to define unique values here
enum msg_id
{
	e_ir_cmd, // infrared command received
	e_last	
};


// message packet
struct msg
{
	enum msg_id id; // which message is this
	uint8_t data[3]; // content depends on ID
};


// wait for a message
struct msg msg_get(void);


/* I unfortunately have to publish this, since I'm inlining msg_post() */

#define MSG_QUEUESIZE 8

#define MSG_MASK (MSG_QUEUESIZE - 1)
#if (MSG_QUEUESIZE & MSG_MASK)
	#error MSG_QUEUESIZE is not a power of 2
#endif


extern struct msg msg_queue[MSG_QUEUESIZE];
extern volatile uint8_t msg_head;
extern volatile uint8_t msg_tail;



// post a message, also allowed from interrupt context
// inline, else we get way more push/pop overhead in ISRs,
//  since this is usually the only external fuction called by an ISR.
static inline void msg_post(struct msg* message)
{
	uint8_t tmphead;

	tmphead = (msg_head + 1) & MSG_MASK;

	msg_head = tmphead; // store new index
	msg_queue[tmphead] = *message; // store received message in queue
}


// post a message to the head, *not* allowed from interrupt context
void msg_post_front(struct msg* message);




#endif // #ifndef _MSG_H
